@@ -91,29 +91,53 @@ impl Display for FromStringConversionError {
     }
 }
 
-pub enum ShiftError {
+pub enum LeftShiftError {
     NegativeShift,
     OutOfMemory,
     TooLarge,
 }
 
-impl ShiftError {
+impl LeftShiftError {
     fn description(&self) -> String {
         match self {
-            ShiftError::NegativeShift => String::from("Shift by negative step is undefined."),
-            ShiftError::OutOfMemory => String::from("Not enough memory for shift result."),
-            ShiftError::TooLarge => String::from("Too large shift step."),
+            LeftShiftError::NegativeShift => String::from("Shift by negative step is undefined."),
+            LeftShiftError::OutOfMemory => String::from("Not enough memory for shift result."),
+            LeftShiftError::TooLarge => String::from("Too large shift step."),
         }
     }
 }
 
-impl Debug for ShiftError {
+impl Debug for LeftShiftError {
     fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
         formatter.write_str(&self.description())
     }
 }
 
-impl Display for ShiftError {
+impl Display for LeftShiftError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        Display::fmt(&self.description(), formatter)
+    }
+}
+
+pub enum RightShiftError {
+    NegativeShift,
+}
+
+impl RightShiftError {
+    fn description(&self) -> String {
+        match self {
+            RightShiftError::NegativeShift => String::from("Shift by negative step is undefined."),
+        }
+    }
+}
+
+impl Debug for RightShiftError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        formatter.write_str(&self.description())
+    }
+}
+
+impl Display for RightShiftError {
     fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
         Display::fmt(&self.description(), formatter)
     }
@@ -173,7 +197,7 @@ impl<Digit: FromStrDigit, const SEPARATOR: char, const SHIFT: usize>
         Self::parse_digits(characters, base).map(|digits| {
             let digits = digits_to_binary_base::<u8, Digit, SHIFT>(&digits, base as usize);
             Self {
-                sign: sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                sign: sign * to_digits_sign(&digits),
                 digits,
             }
         })
@@ -365,7 +389,7 @@ where
             negate_digits(&mut bytes);
             -Sign::one()
         } else {
-            (bytes.len() > 1 || !bytes[0].is_zero()) as Sign
+            to_digits_sign(&bytes)
         };
         Self {
             sign,
@@ -568,14 +592,11 @@ impl<Digit: DivisibleDigit, const SEPARATOR: char, const SHIFT: usize> CheckedDi
                 div_rem_two_or_more_digits::<Digit, SHIFT>(&self.digits, &divisor.digits);
             Some((
                 Self {
-                    sign: self.sign
-                        * divisor.sign
-                        * ((quotient_digits.len() > 1 || !quotient_digits[0].is_zero()) as Sign),
+                    sign: self.sign * divisor.sign * to_digits_sign(&quotient_digits),
                     digits: quotient_digits,
                 },
                 Self {
-                    sign: self.sign
-                        * ((remainder_digits.len() > 1 || !remainder_digits[0].is_zero()) as Sign),
+                    sign: self.sign * to_digits_sign(&remainder_digits),
                     digits: remainder_digits,
                 },
             ))
@@ -1338,11 +1359,11 @@ impl<Digit: EuclidDivisibleDigit, const SEPARATOR: char, const SHIFT: usize> Rem
 impl<Digit: LeftShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShl
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
-    type Output = Result<Self, ShiftError>;
+    type Output = Result<Self, LeftShiftError>;
 
     fn checked_shl(self, shift: Self) -> Self::Output {
         if shift.is_negative() {
-            Err(ShiftError::NegativeShift)
+            Err(LeftShiftError::NegativeShift)
         } else if self.is_zero() {
             Ok(self)
         } else {
@@ -1352,16 +1373,16 @@ impl<Digit: LeftShiftableDigit, const SEPARATOR: char, const SHIFT: usize> Check
                 });
             let shift_quotient =
                 checked_reduce_digits::<Digit, usize, SHIFT>(&shift_quotient_digits)
-                    .ok_or(ShiftError::TooLarge)?;
+                    .ok_or(LeftShiftError::TooLarge)?;
             if shift_quotient >= usize::MAX / size_of::<Digit>() {
-                Err(ShiftError::TooLarge)
+                Err(LeftShiftError::TooLarge)
             } else {
-                let digits = left_shift_digits::<Digit, SHIFT>(
+                let digits = shift_digits_left::<Digit, SHIFT>(
                     &self.digits,
                     shift_quotient,
                     shift_remainder,
                 )
-                .ok_or(ShiftError::OutOfMemory)?;
+                .ok_or(LeftShiftError::OutOfMemory)?;
                 Ok(Self {
                     sign: self.sign,
                     digits,
@@ -1376,26 +1397,26 @@ macro_rules! plain_signed_checked_shl_impl {
         impl<Digit: LeftShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShl<$t>
             for BigInt<Digit, SEPARATOR, SHIFT>
         {
-            type Output = Result<Self, ShiftError>;
+            type Output = Result<Self, LeftShiftError>;
 
             fn checked_shl(self, shift: $t) -> Self::Output {
                 debug_assert!(usize::BITS < <$t>::BITS || SHIFT < <$t>::MAX as usize);
                 if shift.is_negative() {
-                    Err(ShiftError::NegativeShift)
+                    Err(LeftShiftError::NegativeShift)
                 } else if self.is_zero() {
                     Ok(self)
                 } else {
                     let (shift_quotient, shift_remainder) = shift.div_rem(SHIFT as $t);
                     if (<$t>::BITS as usize) + 8 * size_of::<Digit>() >= (usize::BITS as usize)
                         && shift_quotient >= ((usize::MAX / size_of::<Digit>()) as $t) {
-                        Err(ShiftError::TooLarge)
+                        Err(LeftShiftError::TooLarge)
                     } else {
-                        let digits = left_shift_digits::<Digit, SHIFT>(
+                        let digits = shift_digits_left::<Digit, SHIFT>(
                             &self.digits,
                             shift_quotient as usize,
                             unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
                         )
-                        .ok_or(ShiftError::OutOfMemory)?;
+                        .ok_or(LeftShiftError::OutOfMemory)?;
                         Ok(Self {
                             sign: self.sign,
                             digits,
@@ -1414,7 +1435,7 @@ macro_rules! plain_unsigned_checked_shl_impl {
         impl<Digit: LeftShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShl<$t>
             for BigInt<Digit, SEPARATOR, SHIFT>
         {
-            type Output = Result<Self, ShiftError>;
+            type Output = Result<Self, LeftShiftError>;
 
             fn checked_shl(self, shift: $t) -> Self::Output {
                 debug_assert!(usize::BITS < <$t>::BITS || SHIFT < <$t>::MAX as usize);
@@ -1424,14 +1445,14 @@ macro_rules! plain_unsigned_checked_shl_impl {
                     let (shift_quotient, shift_remainder) = shift.div_rem(SHIFT as $t);
                     if (<$t>::BITS as usize) + 8 * size_of::<Digit>() >= (usize::BITS as usize)
                         && shift_quotient >= ((usize::MAX / size_of::<Digit>()) as $t) {
-                        Err(ShiftError::TooLarge)
+                        Err(LeftShiftError::TooLarge)
                     } else {
-                        let digits = left_shift_digits::<Digit, SHIFT>(
+                        let digits = shift_digits_left::<Digit, SHIFT>(
                             &self.digits,
                             shift_quotient as usize,
                             unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
                         )
-                        .ok_or(ShiftError::OutOfMemory)?;
+                        .ok_or(LeftShiftError::OutOfMemory)?;
                         Ok(Self {
                             sign: self.sign,
                             digits,
@@ -1448,11 +1469,11 @@ plain_unsigned_checked_shl_impl!(u8 u16 u32 u64 u128 usize);
 impl<Digit: RightShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShr
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
-    type Output = Result<Self, ShiftError>;
+    type Output = Result<Self, RightShiftError>;
 
     fn checked_shr(self, shift: Self) -> Self::Output {
         if shift.is_negative() {
-            Err(ShiftError::NegativeShift)
+            Err(RightShiftError::NegativeShift)
         } else if self.is_zero() {
             Ok(self)
         } else {
@@ -1462,28 +1483,32 @@ impl<Digit: RightShiftableDigit, const SEPARATOR: char, const SHIFT: usize> Chec
                 });
             let shift_quotient =
                 checked_reduce_digits::<Digit, usize, SHIFT>(&shift_quotient_digits)
-                    .ok_or(ShiftError::TooLarge)?;
+                    .unwrap_or(usize::MAX / size_of::<Digit>());
             if shift_quotient >= usize::MAX / size_of::<Digit>() {
-                Err(ShiftError::TooLarge)
+                Ok(if self.is_negative() {
+                    !Self::zero()
+                } else {
+                    Self::zero()
+                })
             } else if self.is_negative() {
                 let inverted = !self;
-                let digits = right_shift_digits::<Digit, SHIFT>(
+                let digits = shift_digits_right::<Digit, SHIFT>(
                     &inverted.digits,
                     shift_quotient,
                     shift_remainder,
                 );
                 Ok(!Self {
-                    sign: inverted.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                    sign: inverted.sign * to_digits_sign(&digits),
                     digits,
                 })
             } else {
-                let digits = right_shift_digits::<Digit, SHIFT>(
+                let digits = shift_digits_right::<Digit, SHIFT>(
                     &self.digits,
                     shift_quotient,
                     shift_remainder,
                 );
                 Ok(Self {
-                    sign: self.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                    sign: self.sign * to_digits_sign(&digits),
                     digits,
                 })
             }
@@ -1496,12 +1521,12 @@ macro_rules! plain_signed_checked_shr_impl {
         impl<Digit: RightShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShr<$t>
             for BigInt<Digit, SEPARATOR, SHIFT>
         {
-            type Output = Result<Self, ShiftError>;
+            type Output = Result<Self, RightShiftError>;
 
             fn checked_shr(self, shift: $t) -> Self::Output {
                 debug_assert!(usize::BITS < <$t>::BITS || SHIFT < <$t>::MAX as usize);
                 if shift.is_negative() {
-                    Err(ShiftError::NegativeShift)
+                    Err(RightShiftError::NegativeShift)
                 } else if self.is_zero() {
                     Ok(self)
                 } else {
@@ -1509,26 +1534,26 @@ macro_rules! plain_signed_checked_shr_impl {
                     if (<$t>::BITS as usize) + 8 * size_of::<Digit>() >= (usize::BITS as usize)
                         && shift_quotient >= ((usize::MAX / size_of::<Digit>()) as $t)
                     {
-                        Err(ShiftError::TooLarge)
+                        Ok(Self::zero())
                     } else if self.is_negative() {
                         let inverted = !self;
-                        let digits = right_shift_digits::<Digit, SHIFT>(
+                        let digits = shift_digits_right::<Digit, SHIFT>(
                             &inverted.digits,
                             shift_quotient as usize,
                             unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
                         );
                         Ok(!Self {
-                            sign: inverted.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                            sign: inverted.sign * to_digits_sign(&digits),
                             digits,
                         })
                     } else {
-                        let digits = right_shift_digits::<Digit, SHIFT>(
+                        let digits = shift_digits_right::<Digit, SHIFT>(
                             &self.digits,
                             shift_quotient as usize,
                             unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
                         );
                         Ok(Self {
-                            sign: self.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                            sign: self.sign * to_digits_sign(&digits),
                             digits,
                         })
                     }
@@ -1545,7 +1570,7 @@ macro_rules! plain_unsigned_checked_shr_impl {
         impl<Digit: RightShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShr<$t>
             for BigInt<Digit, SEPARATOR, SHIFT>
         {
-            type Output = Result<Self, ShiftError>;
+            type Output = Result<Self, RightShiftError>;
 
             fn checked_shr(self, shift: $t) -> Self::Output {
                 debug_assert!(usize::BITS < <$t>::BITS || SHIFT < <$t>::MAX as usize);
@@ -1556,15 +1581,15 @@ macro_rules! plain_unsigned_checked_shr_impl {
                     if (<$t>::BITS as usize) + 8 * size_of::<Digit>() >= (usize::BITS as usize)
                         && shift_quotient >= ((usize::MAX / size_of::<Digit>()) as $t)
                     {
-                        Err(ShiftError::TooLarge)
+                        Ok(Self::zero())
                     } else {
-                        let digits = right_shift_digits::<Digit, SHIFT>(
+                        let digits = shift_digits_right::<Digit, SHIFT>(
                             &self.digits,
                             shift_quotient as usize,
                             unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
                         );
                         Ok(Self {
-                            sign: self.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                            sign: self.sign * to_digits_sign(&digits),
                             digits,
                         })
                     }
