@@ -1,22 +1,19 @@
 #![feature(const_float_bits_conv)]
-#![feature(const_fn_trait_bound)]
 #![feature(convert_float_to_int)]
-#![feature(destructuring_assignment)]
-#![feature(option_result_unwrap_unchecked)]
 #![feature(specialization)]
 #![feature(trait_alias)]
-#![feature(try_reserve)]
 
 use std::cmp::Ordering;
 use std::convert::{TryFrom, TryInto};
 
 use pyo3::basic::CompareOp;
-use pyo3::class::PyObjectProtocol;
 use pyo3::exceptions::*;
-use pyo3::prelude::{pyclass, pymethods, pymodule, pyproto, PyModule, PyResult, Python};
+use pyo3::prelude::{pyclass, pymethods, pymodule, PyModule, PyResult, Python};
+use pyo3::type_object::PyTypeObject;
 use pyo3::types::{PyBytes, PyFloat, PyLong, PyString};
-use pyo3::{ffi, AsPyPointer, Py, PyAny, PyErr, PyNativeType, PyRef, ToPyObject};
-use pyo3::{IntoPy, PyNumberProtocol, PyObject};
+use pyo3::{AsPyPointer, Py, PyAny, PyErr, PyRef, ToPyObject};
+use pyo3::{IntoPy, PyObject};
+use pyo3_ffi as ffi;
 
 use crate::traits::{
     Abs, BitLength, CheckedDiv, CheckedDivEuclid, CheckedDivRemEuclid, CheckedPow,
@@ -58,14 +55,15 @@ impl PyInt {
         match _value {
             None => Ok(PyInt(_BigInt::zero())),
             Some(value) => {
-                if _base.is_some() || value.is_instance::<PyString>()? {
+                let py = value.py();
+                if _base.is_some() || value.is_instance(PyString::type_object(py))? {
                     match _BigInt::from_str_radix(value.extract::<&str>()?, _base.unwrap_or(10)) {
                         Ok(value) => Ok(PyInt(value)),
                         Err(reason) => Err(PyValueError::new_err(reason.to_string())),
                     }
-                } else if value.is_instance::<PyInt>()? {
+                } else if value.is_instance(PyInt::type_object(py))? {
                     value.extract::<PyInt>()
-                } else if value.is_instance::<PyFloat>()? {
+                } else if value.is_instance(PyFloat::type_object(py))? {
                     Ok(PyInt(
                         _BigInt::try_from(value.extract::<&PyFloat>()?.value()).map_err(
                             |reason| match reason {
@@ -122,28 +120,186 @@ impl PyInt {
     fn gcd(&self, other: Self) -> PyInt {
         PyInt(self.0.clone().gcd(other.0))
     }
+    fn __abs__(&self) -> PyInt {
+        PyInt(self.0.clone().abs())
+    }
+
+    fn __add__(&self, other: PyInt) -> PyInt {
+        PyInt(self.0.clone() + other.0)
+    }
+
+    fn __and__(&self, other: PyInt) -> PyInt {
+        PyInt(self.0.clone() & other.0)
+    }
+
+    fn __bool__(&self) -> bool {
+        !self.0.is_zero()
+    }
 
     fn __ceil__(slf: PyRef<Self>) -> PyRef<Self> {
         slf
     }
 
+    fn __divmod__(&self, other: PyInt) -> PyResult<(PyInt, PyInt)> {
+        match self.0.clone().checked_div_rem_euclid(other.0) {
+            Some((quotient, remainder)) => Ok((PyInt(quotient), PyInt(remainder))),
+            None => Err(PyZeroDivisionError::new_err(
+                UNDEFINED_DIVISION_ERROR_MESSAGE,
+            )),
+        }
+    }
+
+    fn __float__(&self) -> PyResult<PyObject> {
+        match f64::try_from(self.0.clone()) {
+            Ok(value) => Ok(Python::with_gil(|py| value.into_py(py))),
+            Err(reason) => Err(PyOverflowError::new_err(reason.to_string())),
+        }
+    }
+
     fn __floor__(slf: PyRef<Self>) -> PyRef<Self> {
         slf
     }
 
+    fn __floordiv__(&self, other: PyInt) -> PyResult<PyInt> {
+        match self.0.clone().checked_div_euclid(other.0) {
+            Some(result) => Ok(PyInt(result)),
+            None => Err(PyZeroDivisionError::new_err(
+                UNDEFINED_DIVISION_ERROR_MESSAGE,
+            )),
+        }
+    }
+
     fn __getstate__(&self, py: Python) -> PyObject {
         PyBytes::new(py, &self.0.as_bytes()).to_object(py)
     }
 
+    fn __hash__(&self) -> ffi::Py_hash_t {
+        hash(&self.0) as ffi::Py_hash_t
+    }
+
+    fn __int__(&self) -> PyObject {
+        big_int_to_py_long(&self.0)
+    }
+
+    fn __invert__(&self) -> PyInt {
+        PyInt(!self.0.clone())
+    }
+
+    fn __lshift__(&self, other: PyInt) -> PyResult<PyInt> {
+        self.0
+            .clone()
+            .checked_shl(other.0)
+            .map(PyInt)
+            .map_err(|reason| match reason {
+                big_int::LeftShiftError::NegativeShift => PyValueError::new_err(reason.to_string()),
+                big_int::LeftShiftError::OutOfMemory => PyMemoryError::new_err(reason.to_string()),
+                big_int::LeftShiftError::TooLarge => PyOverflowError::new_err(reason.to_string()),
+            })
+    }
+
+    fn __mod__(&self, other: PyInt) -> PyResult<PyInt> {
+        match self.0.clone().checked_rem_euclid(other.0) {
+            Some(result) => Ok(PyInt(result)),
+            None => Err(PyZeroDivisionError::new_err(
+                UNDEFINED_DIVISION_ERROR_MESSAGE,
+            )),
+        }
+    }
+
+    fn __mul__(&self, other: PyInt) -> PyInt {
+        PyInt(self.0.clone() * other.0)
+    }
+
+    fn __neg__(&self) -> PyInt {
+        PyInt(-self.0.clone())
+    }
+
+    fn __or__(&self, other: PyInt) -> PyInt {
+        PyInt(self.0.clone() | other.0)
+    }
+
+    fn __pow__(&self, exponent: PyInt, divisor: Option<PyInt>) -> PyResult<PyObject> {
+        match divisor {
+            Some(value) => {
+                let is_zero_divisor = value.0.is_zero();
+                match self.0.clone().checked_pow_rem_euclid(exponent.0, value.0) {
+                    Some(value) => Ok(to_py_object(PyInt(value))),
+                    None => Err(PyValueError::new_err(if is_zero_divisor {
+                        "Divisor cannot be zero."
+                    } else {
+                        "Base is not invertible for the given divisor."
+                    })),
+                }
+            }
+            None => Ok({
+                if exponent.0.is_negative() {
+                    to_py_object(match unsafe {
+                        _Fraction::new(self.0.clone(), _BigInt::one()).unwrap_unchecked()
+                    }
+                    .checked_pow(exponent.0)
+                    {
+                        Some(value) => Ok(PyFraction(value)),
+                        None => Err(PyZeroDivisionError::new_err(
+                            UNDEFINED_DIVISION_ERROR_MESSAGE,
+                        )),
+                    }?)
+                } else {
+                    to_py_object(PyInt(self.0.clone().pow(exponent.0)))
+                }
+            }),
+        }
+    }
+
+    fn __repr__(&self) -> String {
+        format!("rithm.Int('{}')", self.0)
+    }
+
+    fn __richcmp__(&self, other: PyInt, op: CompareOp) -> bool {
+        compare(&self.0, &other.0, op)
+    }
+
+    fn __rshift__(&self, other: PyInt) -> PyResult<PyInt> {
+        self.0
+            .clone()
+            .checked_shr(other.0)
+            .map(PyInt)
+            .map_err(|reason| match reason {
+                big_int::RightShiftError::NegativeShift => {
+                    PyValueError::new_err(reason.to_string())
+                }
+            })
+    }
+
     fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
         state.extract::<&PyBytes>(py).map(|py_bytes| {
             self.0 = _BigInt::from_bytes(py_bytes.extract().unwrap());
         })
     }
 
+    fn __str__(&self) -> String {
+        self.0.to_string()
+    }
+
+    fn __sub__(&self, other: PyInt) -> PyInt {
+        PyInt(self.0.clone() - other.0)
+    }
+
+    fn __truediv__(&self, other: PyInt) -> PyResult<PyFraction> {
+        match _Fraction::new(self.0.clone(), other.0) {
+            Some(result) => Ok(PyFraction(result)),
+            None => Err(PyZeroDivisionError::new_err(
+                UNDEFINED_DIVISION_ERROR_MESSAGE,
+            )),
+        }
+    }
+
     fn __trunc__(slf: PyRef<Self>) -> PyRef<Self> {
         slf
     }
+
+    fn __xor__(&self, other: PyInt) -> PyInt {
+        PyInt(self.0.clone() ^ other.0)
+    }
 }
 
 #[inline]
@@ -162,21 +318,23 @@ impl PyFraction {
     #[new]
     fn new(_numerator: Option<&PyAny>, _denominator: Option<PyInt>) -> PyResult<Self> {
         match _denominator {
-            Some(denominator) => match _Fraction::new(
-                _numerator
-                    .and_then(|value| value.extract::<PyInt>().ok())
-                    .map(|value| value.0)
-                    .unwrap_or_else(_BigInt::zero),
-                denominator.0,
-            ) {
-                Some(value) => Ok(PyFraction(value)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+            Some(denominator) => match _numerator {
+                Some(numerator) => {
+                    match _Fraction::new(numerator.extract::<PyInt>()?.0, denominator.0) {
+                        Some(value) => Ok(PyFraction(value)),
+                        None => Err(PyZeroDivisionError::new_err(
+                            UNDEFINED_DIVISION_ERROR_MESSAGE,
+                        )),
+                    }
+                }
+                None => Err(PyTypeError::new_err(
+                    "Numerator should be of type Int, but found None",
                 )),
             },
             None => Ok(PyFraction(match _numerator {
                 Some(value) => {
-                    if value.is_instance::<PyInt>()? {
+                    let py = value.py();
+                    if value.is_instance(PyInt::type_object(py))? {
                         _Fraction::new(value.extract::<PyInt>()?.0, _BigInt::one()).unwrap()
                     } else {
                         _Fraction::try_from(value.extract::<f64>()?).map_err(
@@ -204,6 +362,60 @@ impl PyFraction {
         PyInt(self.0.numerator().clone())
     }
 
+    fn __abs__(&self) -> PyFraction {
+        PyFraction(self.0.clone().abs())
+    }
+
+    fn __add__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyFraction::type_object(py))? {
+            Ok(PyFraction(self.0.clone() + other.extract::<PyFraction>()?.0).into_py(py))
+        } else if other.is_instance(PyInt::type_object(py))? {
+            Ok(PyFraction(self.0.clone() + other.extract::<PyInt>()?.0).into_py(py))
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+    fn __bool__(&self) -> bool {
+        self.numerator().__bool__()
+    }
+
+    fn __float__(&self) -> PyResult<PyObject> {
+        match <_Fraction as TryInto<f64>>::try_into(self.0.clone()) {
+            Ok(value) => Ok(Python::with_gil(|py| value.into_py(py))),
+            Err(reason) => Err(PyOverflowError::new_err(reason.to_string())),
+        }
+    }
+
+    fn __floordiv__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyFraction::type_object(py))? {
+            match self
+                .0
+                .clone()
+                .checked_div_euclid(other.extract::<PyFraction>()?.0)
+            {
+                Some(value) => Ok(PyInt(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else if other.is_instance(PyInt::type_object(py))? {
+            match self
+                .0
+                .clone()
+                .checked_div_euclid(other.extract::<PyInt>()?.0)
+            {
+                Some(value) => Ok(PyInt(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
     fn __getstate__(&self, py: Python) -> PyObject {
         (
             self.numerator().__getstate__(py),
@@ -212,6 +424,159 @@ impl PyFraction {
             .to_object(py)
     }
 
+    fn __mod__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyFraction::type_object(py))? {
+            match self
+                .0
+                .clone()
+                .checked_rem_euclid(other.extract::<PyFraction>()?.0)
+            {
+                Some(value) => Ok(PyFraction(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else if other.is_instance(PyInt::type_object(py))? {
+            match self
+                .0
+                .clone()
+                .checked_rem_euclid(other.extract::<PyInt>()?.0)
+            {
+                Some(value) => Ok(PyFraction(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __mul__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyFraction::type_object(py))? {
+            Ok(PyFraction(self.0.clone() * other.extract::<PyFraction>()?.0).into_py(py))
+        } else if other.is_instance(PyInt::type_object(py))? {
+            Ok(PyFraction(self.0.clone() * other.extract::<PyInt>()?.0).into_py(py))
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __neg__(&self) -> PyFraction {
+        PyFraction(-self.0.clone())
+    }
+
+    fn __pow__(&self, other: PyInt, modulo: Option<PyInt>) -> PyResult<PyObject> {
+        if modulo.is_some() {
+            Ok(Python::with_gil(|py| py.NotImplemented()))
+        } else {
+            match self.0.clone().checked_pow(other.0) {
+                Some(value) => Ok(to_py_object(PyFraction(value))),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        }
+    }
+
+    fn __radd__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyInt::type_object(py))? {
+            Ok(PyFraction(other.extract::<PyInt>()?.0 + self.0.clone()).into_py(py))
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __repr__(&self) -> String {
+        format!(
+            "rithm.Fraction({}, {})",
+            self.numerator().__repr__(),
+            self.denominator().__repr__()
+        )
+    }
+
+    fn __richcmp__(&self, other: &PyAny, op: CompareOp) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyFraction::type_object(py))? {
+            Ok(compare(&self.0, &other.extract::<PyFraction>()?.0, op).into_py(py))
+        } else if other.is_instance(PyInt::type_object(py))? {
+            Ok(compare(&self.0, &other.extract::<PyInt>()?.0, op).into_py(py))
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __rfloordiv__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyInt::type_object(py))? {
+            match other
+                .extract::<PyInt>()?
+                .0
+                .checked_div_euclid(self.0.clone())
+            {
+                Some(value) => Ok(PyInt(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __rmod__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyInt::type_object(py))? {
+            match other
+                .extract::<PyInt>()?
+                .0
+                .checked_rem_euclid(self.0.clone())
+            {
+                Some(value) => Ok(PyFraction(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __rmul__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyInt::type_object(py))? {
+            Ok(PyFraction(other.extract::<PyInt>()?.0 * self.0.clone()).into_py(py))
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __rsub__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyInt::type_object(py))? {
+            Ok(PyFraction(other.extract::<PyInt>()?.0 - self.0.clone()).into_py(py))
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __rtruediv__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyInt::type_object(py))? {
+            match other.extract::<PyInt>()?.0.checked_div(self.0.clone()) {
+                Some(value) => Ok(PyFraction(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
     fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
         state.extract::<(PyObject, PyObject)>(py).and_then(
             |(numerator_state, denominator_state)| {
@@ -224,146 +589,42 @@ impl PyFraction {
             },
         )
     }
-}
 
-#[pyproto]
-impl PyNumberProtocol for PyInt {
-    fn __abs__(&self) -> PyInt {
-        PyInt(self.0.clone().abs())
+    fn __str__(&self) -> String {
+        self.0.to_string()
     }
 
-    fn __add__(lhs: PyInt, rhs: PyInt) -> PyInt {
-        PyInt(lhs.0 + rhs.0)
-    }
-
-    fn __and__(lhs: PyInt, rhs: PyInt) -> PyInt {
-        PyInt(lhs.0 & rhs.0)
-    }
-
-    fn __divmod__(lhs: PyInt, rhs: PyInt) -> PyResult<(PyInt, PyInt)> {
-        match lhs.0.checked_div_rem_euclid(rhs.0) {
-            Some((quotient, remainder)) => Ok((PyInt(quotient), PyInt(remainder))),
-            None => Err(PyZeroDivisionError::new_err(
-                UNDEFINED_DIVISION_ERROR_MESSAGE,
-            )),
+    fn __sub__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyFraction::type_object(py))? {
+            Ok(PyFraction(self.0.clone() - other.extract::<PyFraction>()?.0).into_py(py))
+        } else if other.is_instance(PyInt::type_object(py))? {
+            Ok(PyFraction(self.0.clone() - other.extract::<PyInt>()?.0).into_py(py))
+        } else {
+            Ok(py.NotImplemented())
         }
     }
 
-    fn __float__(&self) -> PyResult<PyObject> {
-        match f64::try_from(self.0.clone()) {
-            Ok(value) => Ok(Python::with_gil(|py| value.into_py(py))),
-            Err(reason) => Err(PyOverflowError::new_err(reason.to_string())),
-        }
-    }
-
-    fn __floordiv__(lhs: PyInt, rhs: PyInt) -> PyResult<PyInt> {
-        match lhs.0.checked_div_euclid(rhs.0) {
-            Some(result) => Ok(PyInt(result)),
-            None => Err(PyZeroDivisionError::new_err(
-                UNDEFINED_DIVISION_ERROR_MESSAGE,
-            )),
-        }
-    }
-
-    fn __int__(&self) -> PyObject {
-        big_int_to_py_long(&self.0)
-    }
-
-    fn __invert__(&self) -> PyInt {
-        PyInt(!self.0.clone())
-    }
-
-    fn __lshift__(lhs: PyInt, rhs: PyInt) -> PyResult<PyInt> {
-        lhs.0
-            .checked_shl(rhs.0)
-            .map(PyInt)
-            .map_err(|reason| match reason {
-                big_int::LeftShiftError::NegativeShift => PyValueError::new_err(reason.to_string()),
-                big_int::LeftShiftError::OutOfMemory => PyMemoryError::new_err(reason.to_string()),
-                big_int::LeftShiftError::TooLarge => PyOverflowError::new_err(reason.to_string()),
-            })
-    }
-
-    fn __mod__(lhs: PyInt, rhs: PyInt) -> PyResult<PyInt> {
-        match lhs.0.checked_rem_euclid(rhs.0) {
-            Some(result) => Ok(PyInt(result)),
-            None => Err(PyZeroDivisionError::new_err(
-                UNDEFINED_DIVISION_ERROR_MESSAGE,
-            )),
-        }
-    }
-
-    fn __mul__(lhs: PyInt, rhs: PyInt) -> PyInt {
-        PyInt(lhs.0 * rhs.0)
-    }
-
-    fn __neg__(&self) -> PyInt {
-        PyInt(-self.0.clone())
-    }
-
-    fn __or__(lhs: PyInt, rhs: PyInt) -> PyInt {
-        PyInt(lhs.0 | rhs.0)
-    }
-
-    fn __pow__(lhs: PyInt, rhs: PyInt, divisor: Option<PyInt>) -> PyResult<PyObject> {
-        match divisor {
-            Some(value) => {
-                let is_zero_divisor = value.0.is_zero();
-                match lhs.0.checked_pow_rem_euclid(rhs.0, value.0) {
-                    Some(value) => Ok(to_py_object(PyInt(value))),
-                    None => Err(PyValueError::new_err(if is_zero_divisor {
-                        "Divisor cannot be zero."
-                    } else {
-                        "Base is not invertible for the given divisor."
-                    })),
-                }
+    fn __truediv__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance(PyFraction::type_object(py))? {
+            match self.0.clone().checked_div(other.extract::<PyFraction>()?.0) {
+                Some(value) => Ok(PyFraction(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
             }
-            None => Ok({
-                if rhs.0.is_negative() {
-                    to_py_object(match unsafe {
-                        _Fraction::new(lhs.0, _BigInt::one()).unwrap_unchecked()
-                    }
-                    .checked_pow(rhs.0)
-                    {
-                        Some(value) => Ok(PyFraction(value)),
-                        None => Err(PyZeroDivisionError::new_err(
-                            UNDEFINED_DIVISION_ERROR_MESSAGE,
-                        )),
-                    }?)
-                } else {
-                    to_py_object(PyInt(lhs.0.pow(rhs.0)))
-                }
-            }),
+        } else if other.is_instance(PyInt::type_object(py))? {
+            match self.0.clone().checked_div(other.extract::<PyInt>()?.0) {
+                Some(value) => Ok(PyFraction(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
         }
     }
-
-    fn __rshift__(lhs: PyInt, rhs: PyInt) -> PyResult<PyInt> {
-        lhs.0
-            .checked_shr(rhs.0)
-            .map(PyInt)
-            .map_err(|reason| match reason {
-                big_int::RightShiftError::NegativeShift => {
-                    PyValueError::new_err(reason.to_string())
-                }
-            })
-    }
-
-    fn __sub__(lhs: PyInt, rhs: PyInt) -> PyInt {
-        PyInt(lhs.0 - rhs.0)
-    }
-
-    fn __truediv__(lhs: PyInt, rhs: PyInt) -> PyResult<PyFraction> {
-        match _Fraction::new(lhs.0, rhs.0) {
-            Some(result) => Ok(PyFraction(result)),
-            None => Err(PyZeroDivisionError::new_err(
-                UNDEFINED_DIVISION_ERROR_MESSAGE,
-            )),
-        }
-    }
-
-    fn __xor__(lhs: PyInt, rhs: PyInt) -> PyInt {
-        PyInt(lhs.0 ^ rhs.0)
-    }
 }
 
 fn hash(value: &_BigInt) -> usize {
@@ -398,263 +659,6 @@ fn hash(value: &_BigInt) -> usize {
     result - ((result == usize::MAX) as usize)
 }
 
-#[pyproto]
-impl PyObjectProtocol for PyInt {
-    fn __bool__(self) -> bool {
-        !self.0.is_zero()
-    }
-
-    fn __hash__(&self) -> ffi::Py_hash_t {
-        hash(&self.0) as ffi::Py_hash_t
-    }
-
-    fn __repr__(&self) -> String {
-        format!("rithm.Int('{}')", self.0)
-    }
-
-    fn __richcmp__(&self, other: PyInt, op: CompareOp) -> bool {
-        compare(&self.0, &other.0, op)
-    }
-
-    fn __str__(&self) -> String {
-        self.0.to_string()
-    }
-}
-
-#[pyproto]
-impl PyNumberProtocol for PyFraction {
-    fn __abs__(&self) -> PyFraction {
-        PyFraction(self.0.clone().abs())
-    }
-
-    fn __add__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
-        let py = rhs.py();
-        if rhs.is_instance::<PyFraction>()? {
-            Ok(PyFraction(lhs.0 + rhs.extract::<PyFraction>()?.0).into_py(py))
-        } else if rhs.is_instance::<PyInt>()? {
-            Ok(PyFraction(lhs.0 + rhs.extract::<PyInt>()?.0).into_py(py))
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __float__(&self) -> PyResult<PyObject> {
-        match <_Fraction as TryInto<f64>>::try_into(self.0.clone()) {
-            Ok(value) => Ok(Python::with_gil(|py| value.into_py(py))),
-            Err(reason) => Err(PyOverflowError::new_err(reason.to_string())),
-        }
-    }
-
-    fn __floordiv__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
-        let py = rhs.py();
-        if rhs.is_instance::<PyFraction>()? {
-            match lhs.0.checked_div_euclid(rhs.extract::<PyFraction>()?.0) {
-                Some(value) => Ok(PyInt(value).into_py(py)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        } else if rhs.is_instance::<PyInt>()? {
-            match lhs.0.checked_div_euclid(rhs.extract::<PyInt>()?.0) {
-                Some(value) => Ok(PyInt(value).into_py(py)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __mod__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
-        let py = rhs.py();
-        if rhs.is_instance::<PyFraction>()? {
-            match lhs.0.checked_rem_euclid(rhs.extract::<PyFraction>()?.0) {
-                Some(value) => Ok(PyFraction(value).into_py(py)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        } else if rhs.is_instance::<PyInt>()? {
-            match lhs.0.checked_rem_euclid(rhs.extract::<PyInt>()?.0) {
-                Some(value) => Ok(PyFraction(value).into_py(py)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __mul__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
-        let py = rhs.py();
-        if rhs.is_instance::<PyFraction>()? {
-            Ok(PyFraction(lhs.0 * rhs.extract::<PyFraction>()?.0).into_py(py))
-        } else if rhs.is_instance::<PyInt>()? {
-            Ok(PyFraction(lhs.0 * rhs.extract::<PyInt>()?.0).into_py(py))
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __neg__(&self) -> PyFraction {
-        PyFraction(-self.0.clone())
-    }
-
-    fn __pow__(lhs: PyFraction, rhs: PyInt, modulo: Option<PyInt>) -> PyResult<PyObject> {
-        if modulo.is_some() {
-            Ok(Python::with_gil(|py| py.NotImplemented()))
-        } else {
-            match lhs.0.checked_pow(rhs.0) {
-                Some(value) => Ok(to_py_object(PyFraction(value))),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        }
-    }
-
-    fn __radd__(&self, other: &PyAny) -> PyResult<PyObject> {
-        let py = other.py();
-        if other.is_instance::<PyInt>()? {
-            Ok(PyFraction(other.extract::<PyInt>()?.0 + self.0.clone()).into_py(py))
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __rfloordiv__(&self, other: &PyAny) -> PyResult<PyObject> {
-        let py = other.py();
-        if other.is_instance::<PyInt>()? {
-            match other
-                .extract::<PyInt>()?
-                .0
-                .checked_div_euclid(self.0.clone())
-            {
-                Some(value) => Ok(PyInt(value).into_py(py)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __rmod__(&self, other: &PyAny) -> PyResult<PyObject> {
-        let py = other.py();
-        if other.is_instance::<PyInt>()? {
-            match other
-                .extract::<PyInt>()?
-                .0
-                .checked_rem_euclid(self.0.clone())
-            {
-                Some(value) => Ok(PyFraction(value).into_py(py)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __rmul__(&self, other: &PyAny) -> PyResult<PyObject> {
-        let py = other.py();
-        if other.is_instance::<PyInt>()? {
-            Ok(PyFraction(other.extract::<PyInt>()?.0 * self.0.clone()).into_py(py))
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __rsub__(&self, other: &PyAny) -> PyResult<PyObject> {
-        let py = other.py();
-        if other.is_instance::<PyInt>()? {
-            Ok(PyFraction(other.extract::<PyInt>()?.0 - self.0.clone()).into_py(py))
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __rtruediv__(&self, other: &PyAny) -> PyResult<PyObject> {
-        let py = other.py();
-        if other.is_instance::<PyInt>()? {
-            match other.extract::<PyInt>()?.0.checked_div(self.0.clone()) {
-                Some(value) => Ok(PyFraction(value).into_py(py)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __sub__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
-        let py = rhs.py();
-        if rhs.is_instance::<PyFraction>()? {
-            Ok(PyFraction(lhs.0 - rhs.extract::<PyFraction>()?.0).into_py(py))
-        } else if rhs.is_instance::<PyInt>()? {
-            Ok(PyFraction(lhs.0 - rhs.extract::<PyInt>()?.0).into_py(py))
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __truediv__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
-        let py = rhs.py();
-        if rhs.is_instance::<PyFraction>()? {
-            match lhs.0.checked_div(rhs.extract::<PyFraction>()?.0) {
-                Some(value) => Ok(PyFraction(value).into_py(py)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        } else if rhs.is_instance::<PyInt>()? {
-            match lhs.0.checked_div(rhs.extract::<PyInt>()?.0) {
-                Some(value) => Ok(PyFraction(value).into_py(py)),
-                None => Err(PyZeroDivisionError::new_err(
-                    UNDEFINED_DIVISION_ERROR_MESSAGE,
-                )),
-            }
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-}
-
-#[pyproto]
-impl PyObjectProtocol for PyFraction {
-    fn __bool__(self) -> bool {
-        self.numerator().__bool__()
-    }
-
-    fn __repr__(&self) -> String {
-        format!(
-            "rithm.Fraction({}, {})",
-            self.numerator().__repr__(),
-            self.denominator().__repr__()
-        )
-    }
-
-    fn __richcmp__(&self, other: &PyAny, op: CompareOp) -> PyResult<PyObject> {
-        let py = other.py();
-        if other.is_instance::<PyFraction>()? {
-            Ok(compare(&self.0, &other.extract::<PyFraction>()?.0, op).into_py(py))
-        } else if other.is_instance::<PyInt>()? {
-            Ok(compare(&self.0, &other.extract::<PyInt>()?.0, op).into_py(py))
-        } else {
-            Ok(py.NotImplemented())
-        }
-    }
-
-    fn __str__(&self) -> String {
-        self.0.to_string()
-    }
-}
-
 fn compare<T: PartialOrd<U>, U>(left: &T, right: &U, op: CompareOp) -> bool {
     match op {
         CompareOp::Eq => left == right,
