@@ -21,4 +21,5 @@ def test_connection_with_builtin(first_with_builtin: IntWithBuiltin,
             first_builtin / second_builtin
     else:
         assert is_equivalent_to_builtin_fraction(
-            result, fractions.Fraction(first_builtin, second_builtin))
+                result, fractions.Fraction(first_builtin, second_builtin)
+        )
